using System;
using System.Collections.Generic;
using System.IO;
using System.Data;
using System.Text;

namespace AutoBroker
{
    public class AgentManager
    {
        private List <IAgent> Agents = null;
        //private IValues StockValues = null;

        private void EraseUnusedAgents()
        {         
            /*for (int i = Agents.Count - 1; i >= 0; i--)
            {
                if (Agents[i].GetUsageCount() < 2)
                {
                    Agents.RemoveAt(i);
                }
            }*/
        }


        private int Clamp(int valueToClamp, int min, int max)
        {
            if (valueToClamp < min) valueToClamp = min;
            if (valueToClamp > max) valueToClamp = max;
            return (valueToClamp);
        }

        public void PerformSimulation(IValues stocks, StreamWriter writer)
        {
            int minimalSession = 45;
            int slowExchange = 1;
            int rapidExchange = 3;

            int splitIndex = (int)(((float)stocks.GetNumValues()) * 0.7f);

            IValues [] dividedData = IValuesUtils.Split(stocks, splitIndex);

            //StockValues = dividedData[0];
            Agents = new List<IAgent>();

            writer.WriteLine("Simple agents:");
            
            SimpleAgent.SetFixingValues(stocks);
            int lastVolume = 0;
            int lastDelta = 0;
            
            for (int session = minimalSession; session < dividedData[0].GetNumValues(); session++)
            {
                if (session > minimalSession)
                {// not to check for the first session
                    int neededChange = (int)Math.Round(((dividedData[0].GetValue(session) / dividedData[0].GetValue(session - 1)) - 1.0f) * 100.0f);

                    /*int neededSellers = (int)Math.Log(Clamp(lastDelta - neededChange+1, 1, 10000),2);
                    int neededBuyers = (int)Math.Log(Clamp(neededChange - lastDelta + 1, 1, 10000), 2);*/

                    int neededSellers = Clamp(lastDelta - neededChange, 0, 100);
                    int neededBuyers = Clamp(neededChange - lastDelta, 0, 100);

                    while (neededSellers>0)
                     {
                        SimpleAgent newSeller = new SimpleAgent();
                        newSeller.SetAgentData(AgentType.Seller, session-1);
                        Agents.Add(newSeller);
                        neededSellers -= rapidExchange;
                    }
                    while (neededBuyers > 0)
                    {
                        SimpleAgent newBuyer = new SimpleAgent();
                        newBuyer.SetAgentData(AgentType.Buyer, session-1);
                        Agents.Add(newBuyer);
                        neededBuyers -= rapidExchange;
                    }
                }

                if (session % 100 == 0)
                {
                    EraseUnusedAgents();
                }

                lastVolume = 0;
                lastDelta = 0;
                foreach (IAgent agent in Agents)
                {
                    AgentDecision decision = agent.GetDecision(session);
                    if (decision == AgentDecision.BuyRapidly)
                    {
                        lastVolume += rapidExchange;
                        lastDelta += rapidExchange;
                    } 
                    else
                    if (decision == AgentDecision.SellRapidly)
                    {
                        lastVolume += rapidExchange;
                        lastDelta -= rapidExchange;
                    } 
                    else
                    if (decision == AgentDecision.BuySlowly)
                    {
                        lastVolume += slowExchange;
                        lastDelta += slowExchange;
                    } 
                    else
                    if (decision == AgentDecision.SellSlowly)
                    {
                        lastVolume += slowExchange;
                        lastDelta -= slowExchange;
                    }
                }
            }

            writer.WriteLine("Agents created during teaching : " + Agents.Count);
            long directionMatches = 0;
            long tested = 0;

            for (int session = dividedData[0].GetNumValues(); session < stocks.GetNumValues(); session++)
            {
                int neededChange = (int)Math.Round(((stocks.GetValue(session) / stocks.GetValue(session - 1)) - 1.0f) * 100.0f);
                writer.WriteLine(neededChange.ToString().PadLeft(8) + " | " + lastDelta.ToString().PadLeft(8));
                //if (neededChange != 0)
                {
                    if (Math.Sign(neededChange) == Math.Sign(lastDelta))
                    {
                        directionMatches++;
                    }
                    tested++;
                }
                /*int neededSellers = (int)Math.Log(Clamp(lastDelta - neededChange+1, 1, 10000),2);
                int neededBuyers = (int)Math.Log(Clamp(neededChange - lastDelta + 1, 1, 10000), 2);*/

                int neededSellers = Clamp(lastDelta - neededChange, 0, 100);
                int neededBuyers = Clamp(neededChange - lastDelta, 0, 100);

                while (neededSellers > 0)
                {
                    SimpleAgent newSeller = new SimpleAgent();
                    newSeller.SetAgentData(AgentType.Seller, session-1);
                    Agents.Add(newSeller);
                    neededSellers -= rapidExchange;
                }
                while (neededBuyers > 0)
                {
                    SimpleAgent newBuyer = new SimpleAgent();
                    newBuyer.SetAgentData(AgentType.Buyer, session-1);
                    Agents.Add(newBuyer);
                    neededBuyers -= rapidExchange;
                }

                if (session % 100 == 0)
                {
                    EraseUnusedAgents();
                }

                lastVolume = 0;
                lastDelta = 0;
                foreach (IAgent agent in Agents)
                {
                    AgentDecision decision = agent.GetDecision(session);
                    if (decision == AgentDecision.BuyRapidly)
                    {
                        lastVolume += rapidExchange;
                        lastDelta += rapidExchange;
                    }
                    else
                        if (decision == AgentDecision.SellRapidly)
                        {
                            lastVolume += rapidExchange;
                            lastDelta -= rapidExchange;
                        }
                        else
                            if (decision == AgentDecision.BuySlowly)
                            {
                                lastVolume += slowExchange;
                                lastDelta += slowExchange;
                            }
                            else
                                if (decision == AgentDecision.SellSlowly)
                                {
                                    lastVolume += slowExchange;
                                    lastDelta -= slowExchange;
                                }
                }
            }
            writer.WriteLine("Total agents created : " + Agents.Count);
            float Accuracy = ((float)directionMatches) / (float)tested;
            writer.WriteLine("Accuracy : " + (Accuracy * 100.0f).ToString("F2") + "%");

        }
    }
}
