using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

namespace AutoBroker
{
    public class TrendRatioTypicalReaction : IComparable
    {
        public static int[] TermLengths = { 1, 2, 3, 5, 7, 10, 20, 50, 100, 250 };
        public float FixingToTrend = 1.0f;
        public float[] TermChanges = new float[TermLengths.Length];

        public void RoundFixing(int decimalPlaces)
        {
            FixingToTrend = (float)Math.Round(FixingToTrend, decimalPlaces);
        }

        public int CompareTo(object obj)
        {
            TrendRatioTypicalReaction other = (TrendRatioTypicalReaction)obj;
            if (FixingToTrend < other.FixingToTrend) return -1;
            if (FixingToTrend > other.FixingToTrend) return 1;
            return 0;
        }

        private static void OutputReactionsInfo(List<TrendRatioTypicalReaction> reactions, StreamWriter writer)
        {
            float[] changes = new float[TrendRatioTypicalReaction.TermLengths.Length];

            foreach (TrendRatioTypicalReaction curReact in reactions)
            {
                for (int i = 0; i < changes.Length; i++)
                {
                    changes[i] += curReact.TermChanges[i];
                }
            }

            for (int i = 0; i < changes.Length; i++)
            {
                changes[i] /= reactions.Count;
            }
            writer.Write("| " + Utils.GetPercentStringFromFloat(reactions[0].FixingToTrend) + " | ");
            writer.Write(Utils.GetLongStringFromLong(reactions.Count) + " | ");
            for (int i = 0; i < changes.Length; i++)
            {
                writer.Write(Utils.GetPercentStringWithDetailFromFloat(changes[i]) + " | ");
            }
            writer.WriteLine();
        }

        internal static void PerformAnalysis()
        {
            int trendLength = 15;
            string methodName = "DeltaSMA";

            StreamWriter writer = File.CreateText(System.Environment.CurrentDirectory + @"\analizy\OdchylTrendu" + methodName + trendLength.ToString() + ".txt");

            List<TrendRatioTypicalReaction> reactions = new List<TrendRatioTypicalReaction>(1000000);

            foreach (object curComboStock in MainForm.GetInstance().comboStocks.Items)
            {
                string stockName = (string)curComboStock;
                StockInfo curStock = MainForm.GetInstance().GetStockInfoForName(stockName);
                if (curStock == null) return;
                IValues stockValues = new StockChartValues(curStock);
                Indicator mainTrend=null;
                if (methodName == "EMA")
                {
                    mainTrend = new EmaIndicator(trendLength, stockValues);
                    writer.WriteLine("The influence of stock value to trend (" + methodName + "-" + trendLength.ToString() + ") ratio on future stock values");
                }
                if (methodName == "SMA")
                {
                    mainTrend = new SmaIndicator(trendLength, stockValues);
                    writer.WriteLine("The influence of stock value to trend (" + methodName + "-" + trendLength.ToString() + ") ratio on future stock values");
                }
                if (methodName == "DeltaSMA")
                {
                    AnyValues values = AnyValues.CreateFromIndicator(new SmaIndicator(trendLength, stockValues));
                    mainTrend = new PastToNowRatioIndicator(1, values);
                    stockValues = values;
                }
                for (int i = 0; i < stockValues.GetNumValues() - TrendRatioTypicalReaction.TermLengths[
                    TrendRatioTypicalReaction.TermLengths.Length - 1]; i++)
                {
                    float mainTrendValue = mainTrend.GetValue(i);
                    float stockValue = stockValues.GetValue(i);
                    if (!float.IsNaN(mainTrendValue))
                    {
                        TrendRatioTypicalReaction newReact = new TrendRatioTypicalReaction();
                        newReact.FixingToTrend = stockValue / mainTrendValue;
                        for (int j = 0; j < TrendRatioTypicalReaction.TermLengths.Length; j++)
                        {
                            newReact.TermChanges[j] = stockValues.GetValue(i + j+1) / stockValue;
                        }
                        reactions.Add(newReact);
                    }
                }
            }
            writer.WriteLine("Date of creation: " + DateTime.Now.ToShortDateString());
            writer.WriteLine();

            writer.Write("|Stock | Number |   ");
            for (int i = 0; i < TrendRatioTypicalReaction.TermLengths.Length; i++)
            {
                writer.Write(" " + TrendRatioTypicalReaction.TermLengths[i].ToString().PadLeft(4) + " |   ");
            }
            writer.WriteLine();
            writer.Write("| to   |  of    |        ");
            for (int i = 0; i < TrendRatioTypicalReaction.TermLengths.Length; i++)
            {
                writer.Write(" |        ");
            }
            writer.WriteLine();
            writer.Write("|trend | cases  |        ");
            for (int i = 0; i < TrendRatioTypicalReaction.TermLengths.Length; i++)
            {
                writer.Write(" |        ");
            }
            writer.WriteLine();
            writer.WriteLine();
            
            foreach (TrendRatioTypicalReaction curReaction in reactions)
            {
                curReaction.RoundFixing(2);
            }
            reactions.Sort();
            float lastFixingToTrend = (float)Math.Round(reactions[0].FixingToTrend, 2);
            int lastIndex = 0;
            for (int i=1; i< reactions.Count; i++)
            {
                float fixingToTrend = (float)Math.Round(reactions[i].FixingToTrend, 2);
                if (((lastFixingToTrend != fixingToTrend))||(i==reactions.Count-1))
                {
                    OutputReactionsInfo(reactions.GetRange(lastIndex, (i - lastIndex)), writer);
                    lastFixingToTrend = fixingToTrend;
                    lastIndex = i;
                }
            }

            writer.WriteLine();            
            writer.WriteLine("Stocks used:");
            foreach (object curComboStock in MainForm.GetInstance().comboStocks.Items)
            {
                string stockName = (string)curComboStock;
                writer.WriteLine(stockName);
            }            
            
            writer.Close();
        }
    }
}
