using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace AutoBroker
{
    public partial class ChartControl : UserControl
    {
        protected int MaxValues = 20;

        protected int StockScale = 1;

        protected int ValueInterval = 30;

        protected IChartValues MainChartValues = null;

        protected SmaIndicator IndicatorSMA5 = null;
        protected SmaIndicator IndicatorSMA7 = null;
        protected SmaIndicator IndicatorSMA10 = null;
        protected SmaIndicator IndicatorSMA15 = null;
        protected SmaIndicator IndicatorSMA30 = null;
        protected SmaIndicator IndicatorSMA45 = null;
        protected SmaIndicator IndicatorSMA60 = null;

        protected RsiIndicator<SmaIndicator> IndicatorRSIonSMA5 = null;
        protected RsiIndicator<SmaIndicator> IndicatorRSIonSMA9 = null;
        protected RsiIndicator<SmaIndicator> IndicatorRSIonSMA14 = null;

        protected RsiIndicator<EmaIndicator> IndicatorRSIonEMA5 = null;
        protected RsiIndicator<EmaIndicator> IndicatorRSIonEMA9 = null;
        protected RsiIndicator<EmaIndicator> IndicatorRSIonEMA14 = null;

        public IChartValues Chart
        {
            set { MainChartValues = value; AfterSettingChart();  }
            get { return MainChartValues; }
        }

        private void AfterSettingChart()
        {
            StockScale = 1;
            labelScale.Text = StockScale.ToString() + "x";
            IndicatorSMA5 = new SmaIndicator(5, MainChartValues);
            IndicatorSMA7 = new SmaIndicator(7, MainChartValues);
            IndicatorSMA10 = new SmaIndicator(10, MainChartValues);
            IndicatorSMA15 = new SmaIndicator(15, MainChartValues);
            IndicatorSMA30 = new SmaIndicator(30, MainChartValues);
            IndicatorSMA45 = new SmaIndicator(45, MainChartValues);
            IndicatorSMA60 = new SmaIndicator(60, MainChartValues);

            IndicatorRSIonSMA5 = new RsiIndicator<SmaIndicator>(5, MainChartValues);
            IndicatorRSIonSMA9 = new RsiIndicator<SmaIndicator>(9, MainChartValues);
            IndicatorRSIonSMA14 = new RsiIndicator<SmaIndicator>(14, MainChartValues);

            IndicatorRSIonEMA5 = new RsiIndicator<EmaIndicator>(5, MainChartValues);
            IndicatorRSIonEMA9 = new RsiIndicator<EmaIndicator>(9, MainChartValues);
            IndicatorRSIonEMA14 = new RsiIndicator<EmaIndicator>(14, MainChartValues);
        
        }

        private Bitmap BackBuffer = null;
        public ChartControl()
        {
            InitializeComponent();
            BackBuffer = new Bitmap(panelMainChart.Width, panelMainChart.Height);
        }

        private void ChartControl_Load(object sender, EventArgs e)
        {
        }

        private const int ChartBorders = 8;
        private const int ChartRightBorder = 70;
        private const int ChartDownBorder = 64;

        private void ChartControl_Paint(object sender, PaintEventArgs e)
        {
        }       

        private void panelMainChart_Paint(object sender, PaintEventArgs e)
        {
            if (BackBuffer == null) return;
            Graphics graph = Graphics.FromImage(BackBuffer);
            graph.FillRectangle(new SolidBrush(Color.White), 0, 0, BackBuffer.Width, BackBuffer.Height);
            
            Pen mainPen = new Pen(Color.Green);
            Pen lightPen = new Pen(Color.LightGreen);
            Pen grayPen = new Pen(Color.FromArgb(210,210,210));
            if (MainChartValues != null)
            {
                int valuesNum = Math.Min(MaxValues * StockScale, MainChartValues.GetNumValues());

                float startX = ChartBorders;
                float endX = panelMainChart.Width - ChartBorders - ChartRightBorder;
                float startY = ChartBorders;
                float endY = this.panelMainChart.Height - ChartBorders - ChartDownBorder;

                float rawMinVal, rawMaxVal;
                MainChartValues.GetMinMaxValueForLast(valuesNum, out rawMinVal, out rawMaxVal);
                float minVal = rawMinVal;
                float maxVal = rawMaxVal;
                if (minVal == maxVal)
                {
                    maxVal = minVal + 0.5f;
                    minVal -= 0.5f;
                }

                Point thisPoint = new Point(-1, -1);
                List<Point> chartPoints = new List<Point>();
                List<DateTime> timePoints = new List<DateTime>();

                int itemNum = MainChartValues.GetNumValues() - 1;
                for (int i = valuesNum; i > 0; i -= 1)
                {
                    thisPoint.X = (int)(((float)i) / (float)valuesNum * (endX - startX) + startX);
                    thisPoint.Y = (int)(endY - (MainChartValues.GetValue(itemNum) - minVal) / (maxVal - minVal) * (endY - startY));

                    chartPoints.Add(thisPoint);
                    timePoints.Add(MainChartValues.GetDate(itemNum));                   
                    
                    itemNum -= 1;
                }
                int lastTimePoint = int.MaxValue;
                // Create font and brush.
                Font drawFont = new Font("Arial", 8);
                SolidBrush fontBrush = new SolidBrush(Color.Black);

                if (chartPoints.Count > 1)
                {                    
                    int timeNum = 0;
                    foreach (Point valPoint in chartPoints)
                    {
                        Point startPoint = valPoint;
                        Point endPoint = valPoint;
                        startPoint.Y = (int)startY;
                        endPoint.Y = (int)endY;
                        if (StockScale < 8)
                        {
                            graph.DrawLine(grayPen, startPoint, endPoint);
                        }
                        if (valPoint.X < lastTimePoint - drawFont.Height + 2)
                        {
                            lastTimePoint = valPoint.X;
                            PointF drawPoint = new PointF(valPoint.X - drawFont.Height / 2 + 6 , endY - 12);
                            PointF rotPoint = new PointF(valPoint.X - drawFont.Height / 2, endY);
                            Matrix rotMatrix = new Matrix();
                            rotMatrix.RotateAt(90.0f, rotPoint);
                            graph.Transform = rotMatrix;
                            graph.DrawString(timePoints[timeNum].ToShortDateString(), drawFont, fontBrush, drawPoint);
                            graph.ResetTransform();
                        }
                        timeNum++;
                    }
                }
                graph.DrawString(rawMaxVal.ToString("F2") + " z", drawFont, fontBrush, new PointF(endX + 5.0f, 2));
                graph.DrawString(rawMinVal.ToString("F2") + " z", drawFont, fontBrush, new PointF(endX + 5.0f, endY - drawFont.Height / 2 - 2));
                graph.DrawString(((rawMinVal+rawMaxVal)/2.0f).ToString("F2") + " z", drawFont, fontBrush, new PointF(endX + 5.0f, (startY + endY) / 2 - drawFont.Height / 2));

                Point [] BorderLine = new Point[5];
                BorderLine[0] = new Point((int)startX, (int) startY-1);
                BorderLine[1] = new Point((int)endX, (int) startY-1);
                BorderLine[2] = new Point((int)endX, (int) endY+1);
                BorderLine[3] = new Point((int)startX, (int) endY+1);
                BorderLine[4] = BorderLine[0];
                Pen borderPen = new Pen(Color.DarkGray);
                graph.DrawLines(borderPen, BorderLine);

                if (checkBoxSMA5.Checked)
                {
                    IndicatorSMA5.Draw(graph, Color.FromArgb(120, 0, 0), valuesNum,
                                        startX, startY, endX, endY, minVal, maxVal);
                }
                if (checkBoxSMA7.Checked)
                {
                    IndicatorSMA7.Draw(graph, Color.FromArgb(140, 0, 0), valuesNum,
                                        startX, startY, endX, endY, minVal, maxVal);
                }
                if (checkBoxSMA10.Checked)
                {
                    IndicatorSMA10.Draw(graph,Color.FromArgb(160, 0, 0), valuesNum,
                                        startX, startY, endX, endY, minVal, maxVal);
                }
                if (checkBoxSMA15.Checked)
                {
                    IndicatorSMA15.Draw(graph, Color.FromArgb(180, 0, 0), valuesNum,
                                        startX, startY, endX, endY, minVal, maxVal);
                }
                if (checkBoxSMA30.Checked)
                {
                    IndicatorSMA30.Draw(graph, Color.FromArgb(205, 0, 0), valuesNum,
                                        startX, startY, endX, endY, minVal, maxVal);
                }
                if (checkBoxSMA45.Checked)
                {
                     IndicatorSMA45.Draw(graph, Color.FromArgb(225, 0, 0), valuesNum,
                                        startX, startY, endX, endY, minVal, maxVal);
                }
                if (checkBoxSMA60.Checked)
                {
                    IndicatorSMA60.Draw(graph, Color.FromArgb(255, 0, 0), valuesNum,
                                        startX, startY, endX, endY, minVal, maxVal);
                }                

                Point[] arrPoints = new Point[chartPoints.Count];
                chartPoints.CopyTo(arrPoints);
                if (arrPoints.Length > 1)
                {
                    graph.DrawLines(mainPen, arrPoints);
                }

                borderPen.Dispose();
                drawFont.Dispose();
            }
            mainPen.Dispose();
            grayPen.Dispose();
            lightPen.Dispose();
            e.Graphics.DrawImage(BackBuffer, new Point(0, 0));
        }

        private void buttonZoomIn_Click(object sender, EventArgs e)
        {
            if (StockScale > 1)
            {
                StockScale /= 2;
                RefreshDataVisualization();
                labelScale.Text = StockScale.ToString() + "x";
            }
        }

        private void buttonZoomOut_Click(object sender, EventArgs e)
        {
            if (MainChartValues == null) return;
            if (MainChartValues.GetNumValues() > MaxValues * StockScale)
            {
                StockScale *= 2;
                RefreshDataVisualization();
                labelScale.Text = StockScale.ToString() + "x";
            }
        }

        public void RefreshDataVisualization()
        {
            this.SuspendLayout();
            panelMainChart.Refresh();
            panelVolumes.Refresh();
            this.ResumeLayout();
        }

        private void checkBoxSMA_CheckedChanged(object sender, EventArgs e)
        {
            RefreshDataVisualization();
        }

        private void panelMainChart_Resize(object sender, EventArgs e)
        {
            BackBuffer = new Bitmap(panelMainChart.Width, panelMainChart.Height);
        }

        private void panelVolumes_Paint(object sender, PaintEventArgs e)
        {
            if (MainChartValues == null) return;
            Graphics graph = e.Graphics;
            Pen grayPen = new Pen(Color.FromArgb(210, 210, 210));
            Pen borderPen = new Pen(Color.DarkGray);
            int valuesNum = Math.Min(MaxValues * StockScale, MainChartValues.GetNumValues());

            float startX = ChartBorders;
            float endX = panelVolumes.Width - ChartBorders - ChartRightBorder;
            float startY = ChartBorders;
            float endY = panelVolumes.Height - ChartBorders;

            int rawMinVal, rawMaxVal;
            MainChartValues.GetMinMaxVolumeForLast(valuesNum, out rawMinVal, out rawMaxVal);
            int minVal = rawMinVal;
            int maxVal = rawMaxVal;
            if (minVal == maxVal)
            {
                maxVal = minVal + 1;
            }           

            int itemNum = MainChartValues.GetNumValues() - 1;
            List<Point> volumePoints = new List<Point>();
            Point thisPoint = new Point();
            for (int i = valuesNum; i > 0; i -= 1)
            {
                thisPoint.X = (int)(((float)i) / (float)valuesNum * (endX - startX) + startX);
                thisPoint.Y = (int)(endY - (float)(MainChartValues.GetVolume(itemNum) - minVal) / (float)(maxVal - minVal) * (endY - startY));

                volumePoints.Add(thisPoint);
                itemNum -= 1;
            }
            if (volumePoints.Count >0)
            {   // capping the shape from below
                thisPoint.Y = (int)endY+1;
                volumePoints.Add(thisPoint);
                thisPoint.X = volumePoints[0].X;
                volumePoints.Add(thisPoint);
                thisPoint = volumePoints[0];
                thisPoint.Y = (int)endY + 1;
                volumePoints.Add(thisPoint);
            }
            Brush volumeBrush= new SolidBrush(Color.Red);
            graph.FillPolygon(volumeBrush, volumePoints.ToArray());

            int lineY = (int)((endY - startY) * 0.3f + startY);
            Font rsiFont = new Font("Arial", 7);
            Brush grayBrush = new SolidBrush(Color.DarkGray);
            Brush blackBrush = new SolidBrush(Color.Black);

            graph.DrawString("0", rsiFont, grayBrush, endX + 2, (int)endY - rsiFont.Height / 2);
            graph.DrawString("100", rsiFont, grayBrush, endX + 2, (int)startY - rsiFont.Height / 2);

            graph.DrawString("RSI", rsiFont, grayBrush, endX + 2, (int)(startY + endY)/2 - rsiFont.Height / 2);

            graph.DrawLine(grayPen, (int)startX, lineY, (int)endX, lineY);
            graph.DrawString("70", rsiFont, grayBrush, endX + 2, lineY - rsiFont.Height / 2);
            lineY = (int)((endY - startY) * 0.7f + startY);
            graph.DrawLine(grayPen, (int)startX, lineY, (int)endX, lineY);
            graph.DrawString("30", rsiFont, grayBrush, endX + 2, lineY - rsiFont.Height / 2);

            graph.DrawLine(borderPen, (int)startX, (int)startY, (int)endX, (int)startY);
            graph.DrawLine(borderPen, (int)startX, (int)startY, (int)startX, (int)endY);
            graph.DrawLine(borderPen, (int)startX, (int)endY, (int)endX, (int)endY);
            graph.DrawLine(borderPen, (int)endX, (int)startY, (int)endX, (int)endY);

            Font volFont = new Font("Arial", 7);
            graph.DrawString("Wolumen", rsiFont, blackBrush, endX + 2 + 22, (int)(startY + endY) / 2 - rsiFont.Height / 2);
            graph.DrawString(maxVal.ToString(), rsiFont, blackBrush, endX + 2 + 22, (int)startY - rsiFont.Height / 2);
            graph.DrawString(minVal.ToString(), rsiFont, blackBrush, endX + 2 + 22, (int)endY - rsiFont.Height / 2);
            


            /// RSI on SMA rendering
            if (checkBoxRSIonSMA5.Checked)
            {
                IndicatorRSIonSMA5.Draw(graph, Color.FromArgb(0, 235, 255), valuesNum,
                                    startX, startY, endX, endY, 0, 100);
            }
            if (checkBoxRSIonSMA9.Checked)
            {
                IndicatorRSIonSMA9.Draw(graph, Color.FromArgb(0, 215, 235), valuesNum,
                                    startX, startY, endX, endY, 0, 100);
            }
            if (checkBoxRSIonSMA14.Checked)
            {                
                IndicatorRSIonSMA14.Draw(graph, Color.FromArgb(0, 190, 215), valuesNum,
                                    startX, startY, endX, endY, 0, 100);
            }

            /// RSI on EMA rendering
            if (checkBoxRSIonEMA5.Checked)
            {
                IndicatorRSIonEMA5.Draw(graph, Color.FromArgb(0, 0, 255), valuesNum,
                                    startX, startY, endX, endY, 0, 100);
            }
            if (checkBoxRSIonEMA9.Checked)
            {
                IndicatorRSIonEMA9.Draw(graph, Color.FromArgb(0, 0, 235), valuesNum,
                                    startX, startY, endX, endY, 0, 100);
            }
            if (checkBoxRSIonEMA14.Checked)
            {
                IndicatorRSIonEMA14.Draw(graph, Color.FromArgb(0, 0, 215), valuesNum,
                                    startX, startY, endX, endY, 0, 100);
            }

            blackBrush.Dispose();
            grayBrush.Dispose();
            borderPen.Dispose();
            grayPen.Dispose();
            volumeBrush.Dispose();
        }
    }    
}

