using System;
using System.Globalization;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;

namespace AutoBroker
{
    public partial class ExportDataDetails : Form
    {
        private List<Indicator> indicators = new List<Indicator>();
        private InfoForm infoForm = null;
        private double lastCash = 0.0;
        private int lastDecisionTestLength = 0;
        private int lastDecisionCount = 0;
        private int lastTransactions = 0;
        private int permutationNumber = 0;
        private int lastDecisionIndicatorLength;
        private double lastMinPercent;

        private bool automatedPermutMode = false;
        private int automatedPermutMin = 0;
        private int automatedPermutMax = 0;
        private PermutationGenerator<List<IndicatorDesc>> permutationGenerator = null;
        

        public ExportDataDetails()
        {
            InitializeComponent();
        }

        private int GetDecisionForValue(PastToNowRatioIndicator indicator, int sessionNum, int indicatorLength, int decisionCount, float minPercent)
        {
            if (indicator.GetNumValues() <= indicatorLength + sessionNum) return 0;
            float changeRatio = (indicator.GetValue(sessionNum + indicatorLength)-1.0f) * 100.0f;
            int decision = (int)(changeRatio / minPercent);
            decision += decisionCount/2;
            if (decision < 0) decision = 0;
            if (decision >= decisionCount) decision = decisionCount - 1;
            return decision;
        }

        public void PublishHtml(string filename, string indexFilename)
        {
            StockInfo curStock = MainForm.GetInstance().GetStockInfoForName((string)MainForm.GetInstance().comboStocks.SelectedItem);
            StreamWriter indexWriter = File.AppendText(indexFilename);
            indexWriter.WriteLine(permutationNumber.ToString() + " " + lastCash.ToString());
            indexWriter.Close();

            StreamWriter writer = new StreamWriter(filename, false, UnicodeEncoding.UTF8);
            StringBuilder desc = new StringBuilder(1024);

            desc.AppendLine("<html> <head> <title> Raport z analizy SI </title>");
            desc.AppendLine("<META HTTP-EQUIV=\"Content-type\" CONTENT=\"text/html; charset=ISO-8859-2\">");
            desc.AppendLine("</head>");
            desc.AppendLine("<body bgcolor=\"#66AAFF\">");

            desc.AppendLine("<p><font size=5 color=\"#000000\">");
            desc.AppendLine(@"Raport " + curStock.Name + "/" + permutationNumber.ToString() + " (" + System.DateTime.Now.ToShortDateString() + ")");
            desc.AppendLine(@"<br/></font></p>");
            
            desc.AppendLine("<p><font size=4 face=Arial>Podsumowanie:</font><br>");
            desc.AppendLine("<font size=3 color=\"#000000\" face=\"Courier New\">");
            desc.AppendLine("Pienidze : " + lastCash.ToString() + "<br/>");
            desc.AppendLine("Ilo sesji testowych : " + lastDecisionTestLength.ToString() + "<br/>");
            desc.AppendLine("Ilo transakcji : " + lastTransactions.ToString() + "<br/>");
            desc.AppendLine("</font></p>");

            desc.AppendLine("<p><font size=4 face=Arial>Wskaniki:</font><br>");
            desc.AppendLine("<font size=3 color=\"#000000\" face=\"Courier New\">");
            for (int i = 0; i < this.txtIndicators.Items.Count; i++)
            {
                desc.AppendLine((string)txtIndicators.Items[i] + "<br/>");
            }
            desc.AppendLine("</font></p>");

            desc.AppendLine("<p><font size=4 face=Arial>Ustawienia:</font><br>");
            desc.AppendLine("<font size=3 color=\"#000000\" face=\"Courier New\">");
            desc.AppendLine("Ilo decyzji : " + lastDecisionCount.ToString() + "<br/>");
            desc.AppendLine("Decyzja podejmowana po : " + lastDecisionIndicatorLength.ToString() + " sesjach<br/>");
            desc.AppendLine("Minimalny zysk/strata : " + lastMinPercent.ToString() + "%<br/>");
            desc.AppendLine("</font></p>");

            desc.AppendLine("<p><font size=4 face=Arial>Ustawienia programu zewnetrznego SI:</font><br>");
            desc.AppendLine("<font size=3 color=\"#000000\" face=\"Courier New\">");            
            desc.AppendLine("Ilo koszyczkw : 5 <br/>");
            desc.AppendLine("Szeroko redukcji : 50 <br/>");
            desc.AppendLine("</font></p>");

            desc.AppendLine("<p><font size=4 face=Arial>Reguy:</font><br>");
            desc.AppendLine("<font size=3 color=\"#000000\" face=\"Courier New\">");
            desc.AppendLine(infoForm.GetHtmlText());
            desc.AppendLine(@"<br/></font></p>");

            desc.AppendLine("</body>");
            desc.AppendLine("</html>");
            
            writer.Write(desc.ToString());
            writer.Close();
        }       

        private void CreateSIFiles(int additionalTestCases, bool addPredictionRows)
        {            
            StockInfo curStock = MainForm.GetInstance().GetStockInfoForName((string)MainForm.GetInstance().comboStocks.SelectedItem);
            if (curStock == null) return;

            lastDecisionTestLength = additionalTestCases;

            string baseFilename = "SIexport";
            StreamWriter infoWrite = File.CreateText(baseFilename + ".txt");

            for (int i = 0; i < txtIndicators.Items.Count; i++)
            {
                infoWrite.WriteLine(txtIndicators.Items[i]);
            }
            infoWrite.Close();

            StreamWriter headerWrite = File.CreateText(baseFilename + ".hdr");
            headerWrite.WriteLine(String.Format("# {0:D} conditional attributes + 1 decision", txtIndicators.Items.Count));
            headerWrite.WriteLine();
            for (int i = 0; i < indicators.Count; i++)
            {
                headerWrite.WriteLine("Indicator" + i.ToString() + " conditional, numeric");
            }
            headerWrite.WriteLine("BuyOrSell decision, nominal");
            headerWrite.Close();

            StreamWriter trainWrite = File.CreateText(baseFilename + ".trn");
            StreamWriter testWrite = File.CreateText(baseFilename + ".tst");
            NumberFormatInfo numFormat = (NumberFormatInfo)System.Globalization.CultureInfo.CurrentCulture.NumberFormat.Clone();
            IValues stockValues = new StockChartValues(curStock);
            int decisionIndicatorLength = int.Parse(txtSessionCount.Text);
            lastDecisionIndicatorLength = decisionIndicatorLength;
            float minPercent = float.Parse(this.txtMinChange.Text);
            lastMinPercent = minPercent;
            int decisionCount = int.Parse(txtDecisionCount.Text);

            PastToNowRatioIndicator decisionIndicator = new PastToNowRatioIndicator(decisionIndicatorLength, stockValues);
            numFormat.NumberDecimalSeparator = ".";
            numFormat.NumberDecimalDigits = 3;

            if (indicators.Count > 0)
            {
                int startRow = 0;
                bool anyIndIsNull = true;
                while (anyIndIsNull)
                {
                    if (startRow == indicators[0].GetNumValues()) break;
                    anyIndIsNull = false;
                    for (int i = 0; i < indicators.Count; i++)
                    {
                        anyIndIsNull = anyIndIsNull || (float.IsNaN(indicators[i].GetValue(startRow)));
                    }
                    if (anyIndIsNull) startRow++;
                }

                int trainRows = (indicators[0].GetNumValues() - decisionIndicatorLength) - additionalTestCases;
                int totalRows = indicators[0].GetNumValues();
                if (!addPredictionRows)
                {
                    totalRows -= decisionIndicatorLength;
                }
                if (trainRows > 0)
                {
                    for (int row = startRow; row < trainRows; row++)
                    {
                        for (int i = 0; i < indicators.Count; i++)
                        {
                            if (i > 0)
                            {
                                trainWrite.Write(" ");
                            }
                            trainWrite.Write(indicators[i].GetValue(row).ToString(numFormat));
                        }
                        trainWrite.Write(" ");
                        trainWrite.Write(GetDecisionForValue(decisionIndicator, row, decisionIndicatorLength, decisionCount, minPercent).ToString(numFormat));
                        trainWrite.WriteLine();
                    }
                    for (int row = trainRows; row < totalRows; row++)
                    {
                        for (int i = 0; i < indicators.Count; i++)
                        {
                            if (i > 0)
                            {
                                testWrite.Write(" ");
                            }
                            testWrite.Write(indicators[i].GetValue(row).ToString(numFormat));
                        }
                        testWrite.Write(" ");
                        testWrite.Write(GetDecisionForValue(decisionIndicator, row, decisionIndicatorLength, decisionCount, minPercent).ToString(numFormat));
                        testWrite.WriteLine();
                    }
                }

            }
            trainWrite.Close();
            testWrite.Close();
        }

        private void btnExport_Click(object sender, EventArgs e)
        {
            CreateSIFiles(0, true);
            this.Close();
        }

        private void txtEvaluate_Click(object sender, EventArgs e)
        {
            CreateSIFiles(trackBarSplit.Value, false);
            infoForm = new InfoForm();
            infoForm.Show(this);
            EvaluateByExternalSI();            
        }

        private void EvaluateByExternalSI()
        {
            string javaArgs = @"rseslib.tools.BasicTrainTestTester " +
                                @"D:\ACESPROS\C#\AutoBroker\SIexport.trn " +
                                @"D:\ACESPROS\C#\AutoBroker\SIexport.tst " +
                                @"D:\ACESPROS\C#\AutoBroker\SIexport.hdr";
            Process newProcess = new Process();
            newProcess.StartInfo.FileName = @"java.exe";
            newProcess.StartInfo.Arguments = javaArgs;
            newProcess.StartInfo.UseShellExecute = false;
            newProcess.StartInfo.RedirectStandardOutput = true;
            newProcess.StartInfo.WorkingDirectory = @"D:\ACESPROS\java\Projects\LabSI\bin";
            newProcess.StartInfo.CreateNoWindow = true;
            newProcess.EnableRaisingEvents = true;

            newProcess.OutputDataReceived += new DataReceivedEventHandler(infoForm.ConsoleOutputHandler);
            newProcess.Exited += new EventHandler(OnSiProcessExit);
            newProcess.Start();
            newProcess.BeginOutputReadLine();
        }

        public delegate void AfterEvent();

        private void OnSiProcessExit(Object sender, EventArgs e)
        {
            AfterEvent d = new AfterEvent(DirectlyProcessDeicisions);
            this.Invoke(d, null);
        }

        private void DirectlyProcessDeicisions()
        {
            if (File.Exists(@"D:\ACESPROS\C#\AutoBroker\SIresults.txt"))
            {
                File.Delete(@"D:\ACESPROS\C#\AutoBroker\SIresults.txt");
            }
            try
            {
                File.Move(@"D:\ACESPROS\java\Projects\LabSI\bin\SIresults.txt",
                            @"D:\ACESPROS\C#\AutoBroker\SIresults.txt");

                StreamReader reader = File.OpenText(@"D:\ACESPROS\C#\AutoBroker\SIresults.txt");
                List<int> decisions = new List<int>();
                while (!reader.EndOfStream)
                {
                    int decision = int.Parse(reader.ReadLine());
                    decisions.Add(decision);
                }
                reader.Close();
                lastDecisionCount = int.Parse(txtDecisionCount.Text);
                lastCash = ProcessChosenDecisions(decisions, MainForm.GetInstance().GetStockInfoForName((string)MainForm.GetInstance().comboStocks.SelectedItem).Fixing, lastDecisionCount);
                StockInfo curStock = MainForm.GetInstance().GetStockInfoForName((string)MainForm.GetInstance().comboStocks.SelectedItem);
                if (!Directory.Exists(@"analizy\SI\" + curStock.Name))
                {
                    Directory.CreateDirectory(@"analizy\SI\" + curStock.Name);
                }
                string htmlFile = @"analizy\SI\" + curStock.Name + @"\raport" + String.Format("{0:D}", permutationNumber) + ".htm";
                string indexFile = @"analizy\SI\" + curStock.Name + @"\index.dat";
                PublishHtml(htmlFile, indexFile);
                if (!automatedPermutMode)
                {
                    MessageBox.Show("Ze 100 z uzyskano " + lastCash + "z " + "(bez kosztw prowizji). " +
                                    "Przeprowadzono " + lastTransactions + " transakcji. " +
                                    "Raport zosta zapisany w pliku " + htmlFile + ".",
                                    "Analiza SI zakoczona");
                }
            }
            catch
            {
                MessageBox.Show("Nie zapisano raporu", "Bd w analizie SI");
            }                        
            if (automatedPermutMode)
            {
                AutomatedSearchIteration();
            }
            else
            {
                infoForm.Close();
            }
        }

        public double ProcessChosenDecisions(List<int> decisions, List<StockValue> stockValues, int decisionTypeCount)
        {
            
            double cash = 100.0;
            double shares = 0.0;
            int startDecision = Math.Max(0, decisions.Count - stockValues.Count);
            int startValue = Math.Max(0, stockValues.Count - decisions.Count);
            int transactions = 0;

            if (decisionTypeCount == 3)
            {
                while (startDecision < decisions.Count)
                {

                    switch (decisions[startDecision])
                    {
                        case 0:
                            if (shares > 0.0)
                            {
                                cash += shares * stockValues[startValue].CloseValue.ToFloat();
                                shares = 0;
                                transactions++;
                            }
                            break;
                        case 2:
                            if (cash > 0.0)
                            {
                                shares += (cash / stockValues[startValue].CloseValue.ToFloat());
                                cash = 0;
                                transactions++;
                            }
                            break;
                    }
                    startDecision++;
                    startValue++;                    
                }                
            }
            else
                if (decisionTypeCount == 5)
                {
                    while (startDecision < decisions.Count)
                    {

                        switch (decisions[startDecision])
                        {
                            case 0:
                                if (shares > 0.0)
                                {
                                    cash += shares * stockValues[startValue].CloseValue.ToFloat();
                                    shares = 0;
                                    transactions++;
                                }
                                break;
                            case 1:
                                if (shares > 0.0)
                                {
                                    cash += (shares / 2.0) * stockValues[startValue].CloseValue.ToFloat();
                                    shares /= 2.0;
                                    transactions++;
                                }
                                break;
                            case 3:
                                if (cash > 0.0)
                                {
                                    shares += ((cash / 2.0) / stockValues[startValue].CloseValue.ToFloat());
                                    cash /= 2.0;
                                    transactions++;
                                }
                                break;
                            case 4:
                                if (cash > 0.0)
                                {
                                    shares += (cash / stockValues[startValue].CloseValue.ToFloat());
                                    cash = 0;
                                    transactions++;
                                }
                                break;
                        }
                        startDecision++;
                        startValue++;
                    }
                }
                else
                {
                    MessageBox.Show("Mozna obliczyc wartosc tylko dla 3- i 5-decyzyjnych!");
                }
            lastTransactions = transactions;
            cash += shares * stockValues[stockValues.Count-1].CloseValue.ToFloat();
            return cash;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void ExportDataDetails_Load(object sender, EventArgs e)
        {
            indicatorSelectBox1.SetDetailFrame(this);
            try
            {
                StockInfo curStock = MainForm.GetInstance().GetStockInfoForName((string)MainForm.GetInstance().comboStocks.SelectedItem);
                trackBarSplit.Maximum = curStock.Fixing.Count;
            }
            catch { }
        }

        public void AddIndicator(string description, Indicator indicator)
        {
            indicators.Add(indicator);
            this.txtIndicators.Items.Add(description);
        }

        private void trackBarSplit_Scroll(object sender, EventArgs e)
        {
            lblTestCases.Text = trackBarSplit.Value.ToString();
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (this.txtIndicators.SelectedIndex >= 0)
            {
                indicators.RemoveAt(this.txtIndicators.SelectedIndex);
                this.txtIndicators.Items.RemoveAt(this.txtIndicators.SelectedIndex);
            }
        }

        public void CreateIndicator(string indType, int paramNum, string stockName, ref string descText, ref Indicator newInd)
        {
            descText = indType;
            if (indType != "Cena waloru")
            {
                descText += "-" + paramNum;
            }
            descText += " ";
            descText += stockName;
            StockInfo curStock = MainForm.GetInstance().GetStockInfoForName(stockName);
            if (curStock == null) return;
            IValues stockValues = new StockChartValues(curStock);

            if (indType == "Cena waloru")
            {
                AnyValueIndicator ind = new AnyValueIndicator(stockValues);
                newInd = ind;
            }
            else
                if (indType == "Zmiana ceny")
                {
                    PastToNowRatioIndicator ind = new PastToNowRatioIndicator(paramNum, stockValues);
                    newInd = ind;
                }
                else
                    if (indType == "SMA")
                    {
                        SmaIndicator smaInd = new SmaIndicator(paramNum, stockValues);
                        AnyValueIndicator stockInd = new AnyValueIndicator(stockValues);
                        newInd = new IndicatorRelationIndicator(smaInd, stockInd);
                    }
                    else
                        if (indType == "EMA")
                        {
                            EmaIndicator emaInd = new EmaIndicator(paramNum, stockValues);
                            AnyValueIndicator stockInd = new AnyValueIndicator(stockValues);
                            newInd = new IndicatorRelationIndicator(emaInd, stockInd);
                        }
                        else
                            if (indType == "RSI")
                            {
                                newInd = new RsiIndicator<EmaIndicator>(paramNum, stockValues);
                            }
        }

        private void btnSeek_Click(object sender, EventArgs e)
        {
            SiOptionsDialog options = new SiOptionsDialog();
            options.SetDetails(this);
            DialogResult result = options.ShowDialog(this);
            if (result == DialogResult.OK)
            {
                automatedPermutMin = options.GetMin();
                automatedPermutMax = options.GetMax();

                automatedPermutMode = true;
                permutationNumber = automatedPermutMin - 1;
                permutationGenerator = new PermutationGenerator<List<IndicatorDesc>>(IndicatorPermutFunction, permutationNumber);
                AutomatedSearchIteration();
            }
        }

        private static int [] generatorLengths = {1, 2, 3, 5, 7, 10, 14, 22 };

        private static int GetGeneratorLengthIndex(int length)
        {
            for (int i = 0; i < generatorLengths.Length; ++i)
            {
                if (length == generatorLengths[i]) return i;
            }
            return -1;
        }

        private static string[] generatorTypes = { "Zmiana ceny", "EMA", "RSI" };
        
        private static int GetGeneratorTypeIndex(string typeName)
        {
            for (int i = 0; i < generatorTypes.Length; ++i)
            {
                if (typeName == generatorTypes[i]) return i;
            }
            return -1;
        }

        private void incrementIndicatorPermut(List <IndicatorDesc> lastPerm, int indIndex)
        {
            IndicatorDesc inDesc = lastPerm[indIndex];
            int oldIndex = GetGeneratorLengthIndex(inDesc.ParamNum);
            oldIndex++;
            if (oldIndex >= generatorLengths.Length - (2 - indIndex))
            {
                int indTypeIndex = GetGeneratorTypeIndex(inDesc.IndType);
                indTypeIndex++;

                if (indTypeIndex >= generatorTypes.Length)
                {
                    if (indIndex > 0)
                    {
                        incrementIndicatorPermut(lastPerm, indIndex - 1);
                        oldIndex = GetGeneratorLengthIndex(lastPerm[indIndex - 1].ParamNum) + 1;
                    }                    
                    indTypeIndex = 0;
                }
                if (indIndex > 0)
                {
                    oldIndex = GetGeneratorLengthIndex(lastPerm[indIndex - 1].ParamNum) + 1;
                }
                else
                {
                    oldIndex = 0;
                }
                inDesc.IndType = generatorTypes[indTypeIndex];                
            }
            inDesc.ParamNum = generatorLengths[oldIndex];
        }

        private void IndicatorPermutFunction(int permutationNum, ref List<IndicatorDesc> lastPerm)
        {
            string stockName = (string)MainForm.GetInstance().comboStocks.SelectedItem;
            if (lastPerm == null ) lastPerm = new List<IndicatorDesc>(3);
            if (lastPerm.Count == 0)
            { // first permutation
                for (int i=0; i<3;++i)
                {
                    IndicatorDesc indDesc = new IndicatorDesc();
                    indDesc.IndType = generatorTypes[0];
                    indDesc.ParamNum = generatorLengths[i];
                    indDesc.StockName = stockName;
                    lastPerm.Add(indDesc);
                }
            }
            else
            {
                int changeSecondInterval = generatorLengths.Length * generatorTypes.Length;
                int changeFirstInterval = changeSecondInterval * changeSecondInterval;

                incrementIndicatorPermut(lastPerm, 2);               
            }
        }

        private void AutomatedSearchIteration()
        {
            if (permutationNumber < this.automatedPermutMax)
            {
                permutationNumber++;
                permutationGenerator.NextPermutation();
                List <IndicatorDesc> indPerm = permutationGenerator.GetCurrentPermutation();
                indicators.Clear();
                txtIndicators.Items.Clear();
                for (int i = 0; i < 3; i++)
                {
                    string descText = null;
                    Indicator ind = null;
                    CreateIndicator(indPerm[i].IndType, indPerm[i].ParamNum, indPerm[i].StockName, ref descText, ref ind);
                    indicators.Add(ind);
                    txtIndicators.Items.Add(descText);
                }
                CreateSIFiles(trackBarSplit.Value, false);
                if (infoForm == null)
                {
                    infoForm = new InfoForm();
                }
                else
                {
                    infoForm.ClearText();
                }
                if (!infoForm.Visible)
                {
                    infoForm.Show(this);
                }
                infoForm.Left = this.Width + this.Left;
                infoForm.Top = this.Top;                
                EvaluateByExternalSI();
            }
            else
            {
                automatedPermutMode = false;
                if (infoForm != null)
                {
                    infoForm.Close();
                    infoForm = null;
                }
            }
        }

        private struct IndexPair : IComparable
        {
            public int reportID;
            public double cash;

            public int CompareTo(object obj)
            {
                IndexPair secondPair = (IndexPair)obj;
                if (cash < secondPair.cash) return 1;
                if (cash > secondPair.cash) return -1;
                return 0;
            }
        }

        internal void BuildIndex()
        {            
            StockInfo curStock = MainForm.GetInstance().GetStockInfoForName((string)MainForm.GetInstance().comboStocks.SelectedItem);
            string indexFile = @"analizy\SI\" + curStock.Name + @"\index.dat";
            string outputFile = @"analizy\SI\" + curStock.Name + @"\rindex.htm";

            try
            {
                File.Copy(@"analizy\SI\index.html", @"analizy\SI\" + curStock.Name + @"\index.html");
            }
            catch { }

            StreamReader reader = File.OpenText(indexFile);
            List<IndexPair> pairs = new List<IndexPair>(2000);
            while (!reader.EndOfStream)
            {
                string [] lines = reader.ReadLine().Split(" ".ToCharArray());
                if (lines.Length == 2)
                {
                    IndexPair newPair;
                    newPair.reportID = int.Parse(lines[0]);
                    newPair.cash = double.Parse(lines[1]);
                    pairs.Add(newPair);
                }
            }            
            reader.Close();
            pairs.Sort();
            StringBuilder indexPage = new StringBuilder(30000);
            indexPage.AppendLine("<html> <head> <title> Wyniki raportu SI </title>");
            indexPage.AppendLine("<META HTTP-EQUIV=\"Content-type\" CONTENT=\"text/html; charset=ISO-8859-2\">");
            indexPage.AppendLine("</head>");
            indexPage.AppendLine("<body bgcolor=\"#66AAFF\">");

            indexPage.AppendLine("<TABLE BORDER=\"1\" BGCOLOR=\"#FFFFFF\">");
            indexPage.AppendLine("<TR><TD>Pieniadze</TD><TD>Link</TD></TR>");

            for (int i = 0; i < pairs.Count; ++i)
            {
                indexPage.AppendLine(string.Format("<TR><TD>{0:D}%</TD><TD><a href=\"raport{1:D}.htm\" target=\"left\">{1:D}</a></TD></TR>", (int)pairs[i].cash, pairs[i].reportID));
            }
            indexPage.AppendLine("</TABLE>");
            
            indexPage.AppendLine("</body>");
            indexPage.AppendLine("</html>");
            StreamWriter writer = new StreamWriter(outputFile, false, UnicodeEncoding.UTF8);
            writer.Write(indexPage.ToString());
            writer.Close();
        }

        internal void ShowReport()
        {
            StockInfo curStock = MainForm.GetInstance().GetStockInfoForName((string)MainForm.GetInstance().comboStocks.SelectedItem);            

            string outputFile = @"analizy\SI\" + curStock.Name + @"\rindex.htm";
            Process newProcess = new Process();
            newProcess.StartInfo.FileName = @"analizy\SI\" + curStock.Name + @"\index.html";
            newProcess.StartInfo.UseShellExecute = true;            
            newProcess.Start();
        }

        internal void BuildComparison()
        {
            FolderBrowserDialog folder = new FolderBrowserDialog();
            folder.SelectedPath = @"\analizy";
            if (folder.ShowDialog(this) != DialogResult.OK)
            {
                return;
            }
            string dir = folder.SelectedPath;
            DirectoryInfo dirInfo = new DirectoryInfo(dir);
            DirectoryInfo[] subDirs = dirInfo.GetDirectories();
            if ((subDirs == null) || (subDirs.Length == 0))
            {
                return;
            }
            SortedDictionary<int, double> indicatorPerformance = new SortedDictionary<int,double>();

            for (int i = 0; i < subDirs.Length; ++i)
            {
                StreamReader reader = new StreamReader(subDirs[i].FullName + @"\index.dat");
                while (!reader.EndOfStream)
                {
                    string line = reader.ReadLine();
                    if ((line==null)||(line.Length==0)) continue;
                    string [] words = line.Split(" ".ToCharArray());
                    if ((words == null) || (words.Length != 2)) continue;
                    int indicatorNum = int.Parse(words[0]);
                    double indicatorPerf = double.Parse(words[1]);
                    if (!indicatorPerformance.ContainsKey(indicatorNum))
                    {
                        indicatorPerformance.Add(indicatorNum, 100.0);
                    }
                    indicatorPerformance[indicatorNum] *= indicatorPerf * 0.01;
                }
                reader.Close();
            }

            StringBuilder indexPage = new StringBuilder(30000);
            indexPage.AppendLine("<html> <head> <title> Wyniki raportu SI </title>");
            indexPage.AppendLine("<META HTTP-EQUIV=\"Content-type\" CONTENT=\"text/html; charset=ISO-8859-2\">");
            indexPage.AppendLine("</head>");
            indexPage.AppendLine("<body bgcolor=\"#66AAFF\">");

            indexPage.AppendLine("<TABLE BORDER=\"1\" BGCOLOR=\"#FFFFFF\">");
            indexPage.AppendLine("<TR><TD>Pieniadze</TD><TD>Wskanik</TD></TR>");

            ReversedComparer<double> revComp = new ReversedComparer<double>();
            SortedDictionary<double, int> invertedIndPerf = new SortedDictionary<double, int>(revComp);
            foreach (KeyValuePair<int, double> valPair in indicatorPerformance)
            {
                invertedIndPerf.Add(valPair.Value, valPair.Key);                
            }

            foreach (KeyValuePair<double, int> valPair in invertedIndPerf)
            {
                indexPage.AppendLine("<tr><td>" + ((int)(double)valPair.Key).ToString() + "</td><td> " + valPair.Value.ToString() + "</td></tr>");
            }
            indexPage.AppendLine("</TABLE>");

            indexPage.AppendLine("</body>");
            indexPage.AppendLine("</html>");
            StreamWriter writer = new StreamWriter(dir + @"\comparison.html", false, UnicodeEncoding.UTF8);
            writer.Write(indexPage.ToString());
            writer.Close();
            MessageBox.Show("Zapisano raport");
        }
    }
}