using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace AutoBroker
{

    public class EmaIndicator : AverageIndicator
    {
        public int EmaLength = 1;
        protected List<float> EmaValues = new List<float>();

        public EmaIndicator()
        {
        }

        public EmaIndicator(int averageLength, IValues values)
        {
            EmaLength = averageLength;
            ComputeFor(values);
        }

        protected override void ComputeFor(IValues values)
        {
            if (values == null) return;
            if (EmaLength == 0) return;
            EmaValues = new List<float>();
            if (values.GetNumValues() < EmaLength) return;
            float expFactor = 1.0f - 2.0f / ((float)EmaLength + 1.0f);
            float divideBy = 0.0f;
            float exponent = 1.0f;
            for (int i = 0; i < EmaLength; i++)
            {
                divideBy += exponent;
                exponent *= expFactor;
            }
            for (int i = 0; i < EmaLength -1; i++)
            {
                EmaValues.Add(float.NaN);
            }
            
            for (int i = EmaLength - 1; i < values.GetNumValues(); i++)
            {
                float sum = 0.0f;
                exponent = 1.0f;
                for (int j = i; j > i-EmaLength; j--)
                {
                    sum += values.GetValue(j) * exponent;
                    exponent *= expFactor;
                }
                EmaValues.Add(sum/divideBy);
            }
        }

        public override float GetValue(int index)
        {
            if ((index >= EmaValues.Count) || (index < 0)) return float.NaN;
            return EmaValues[index];
        }

        public override int GetNumValues()
        {
            return EmaValues.Count;
        }

        public void Draw(Graphics graph, Color color, int valuesNum,
                            float startX, float startY, float endX, float endY,
                            float minVal, float maxVal)
        {
            int itemNum = GetNumValues() - 1;
            List<Point> points = new List<Point>();
            Point thisPoint = new Point();
            for (int i = valuesNum; i > 0; i--)
            {
                if (float.IsNaN(GetValue(itemNum)))
                {
                    itemNum--;
                    continue;
                }
                thisPoint.X = (int)(((float)i) / (float)valuesNum * (endX - startX) + startX);
                thisPoint.Y = (int)(endY - (GetValue(itemNum) - minVal) / (maxVal - minVal) * (endY - startY));
                if (thisPoint.Y < (int)startY) thisPoint.Y = (int)startY;
                if (thisPoint.Y > (int)endY) thisPoint.Y = (int)endY;
                points.Add(thisPoint);
                itemNum--;
            }
            if (points.Count == 0) return;
            Point[] arrPoints = new Point[points.Count];
            points.CopyTo(arrPoints);
            Pen colorPen = new Pen(color);
            graph.DrawCurve(colorPen, arrPoints);
            colorPen.Dispose();
        }

        public override void SetAfterNew(int length, IValues values)
        {
            EmaLength = length;
            ComputeFor(values);
        }
    }
}
