using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections.Generic;
using System.Data;
using System.Text;

namespace AutoBroker
{
    public class RsiIndicator<AverageType> : Indicator where AverageType :  AverageIndicator, new()
    {
        public int RsiLength = 1;
        protected List<float> RsiValues = new List<float>();

        public RsiIndicator(int rsiLength, IValues values)
        {
            RsiLength = rsiLength;
            ComputeFor(values);
        }

        protected override void ComputeFor(IValues values)
        {
            if ((values==null) || (values.GetNumValues() < 1)) return;
            AnyValues upAverage = new AnyValues();
            AnyValues downAverage = new AnyValues();
            upAverage.AddValue(new ValueRep(0.0f));
            downAverage.AddValue(new ValueRep(0.0f));
            float lastValue = values.GetValue(0);
            for (int i = 1; i < values.GetNumValues(); i++)
            {
                float priceChange = values.GetValue(i) - lastValue;
                if (priceChange >= 0.0f)
                {
                    upAverage.AddValue(new ValueRep(priceChange));
                    downAverage.AddValue(new ValueRep(0.0f));
                }
                else
                {
                    upAverage.AddValue(new ValueRep(0.0f));
                    downAverage.AddValue(new ValueRep(-priceChange));
                }
                lastValue = values.GetValue(i);
            }
            AverageType upTrendIndicator = new AverageType();
            upTrendIndicator.SetAfterNew(RsiLength, upAverage);

            AverageType downTrendIndicator = new AverageType();
            downTrendIndicator.SetAfterNew(RsiLength, downAverage);
            for (int i = 0; i < values.GetNumValues(); i++)
            {
                if (downTrendIndicator.GetValue(i) < 0.001f)
                {   // to avoid adding NaN
                    RsiValues.Add(100.0f);
                }
                else
                {
                    RsiValues.Add(100.0f - (100.0f / (1.0f + (upTrendIndicator.GetValue(i) / downTrendIndicator.GetValue(i)))));
                }
            }
        }

        public override float GetValue(int index)
        {
            return RsiValues[index];
        }

        public override int GetNumValues()
        {
            return RsiValues.Count;
        }

        public void Draw(Graphics graph, Color color, int valuesNum,
                            float startX, float startY, float endX, float endY,
                            float minVal, float maxVal)
        {
            int itemNum = GetNumValues() - 1;
            List<Point> points = new List<Point>();
            Point thisPoint = new Point();
            for (int i = valuesNum; i > 0; i--)
            {
                if (float.IsNaN(GetValue(itemNum)))
                {
                    itemNum--;
                    continue;
                }
                thisPoint.X = (int)(((float)i) / (float)valuesNum * (endX - startX) + startX);
                thisPoint.Y = (int)(endY - (GetValue(itemNum) - minVal) / (maxVal - minVal) * (endY - startY));
                if (thisPoint.Y < (int)startY) thisPoint.Y = (int)startY;
                if (thisPoint.Y > (int)endY) thisPoint.Y = (int)endY;
                points.Add(thisPoint);
                itemNum--;
            }
            if (points.Count == 0) return;
            Point[] arrPoints = new Point[points.Count];
            points.CopyTo(arrPoints);
            Pen colorPen = new Pen(color);
            graph.DrawLines(colorPen, arrPoints);
            colorPen.Dispose();
        }
    }
}
