using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace AutoBroker
{

    public class SmaIndicator : AverageIndicator
    {
        public int SmaLength = 1;
        protected List<float> SmaValues = new List<float>();

        public SmaIndicator()
        {
        }

        public SmaIndicator(int averageLength, IValues values)
        {
            SmaLength = averageLength;
            ComputeFor(values);
        }

        protected override void ComputeFor(IValues values)
        {
            if (values == null) return;
            SmaValues = new List<float>();
            if (values.GetNumValues() < SmaLength) return;
            long sum = 0;
            for (int i = 0; i < SmaLength - 1; i++)
            {
                sum += values.GetDecIntValue(i);
                SmaValues.Add(float.NaN);
            }
            sum += values.GetDecIntValue(SmaLength - 1);
            SmaValues.Add(((float)sum) / (100.0f * (float)SmaLength));
            for (int i = SmaLength; i < values.GetNumValues(); i++)
            {
                sum -= values.GetDecIntValue(i - SmaLength);
                sum += values.GetDecIntValue(i);
                SmaValues.Add(((float)sum) / (100.0f * (float)SmaLength));
            }
        }

        public override float GetValue(int index)
        {
            if ((index >= SmaValues.Count) || (index < 0)) return float.NaN;
            return SmaValues[index];
        }

        public override int GetNumValues()
        {
            return SmaValues.Count;
        }

        public void Draw(  Graphics graph, Color color, int valuesNum,
                            float startX, float startY, float endX, float endY,
                            float minVal, float maxVal)
        {
            int itemNum = GetNumValues() - 1;
            List<Point> points = new List<Point>();
            Point thisPoint = new Point();
            for (int i = valuesNum; i > 0; i--)
            {
                if (float.IsNaN(GetValue(itemNum)))
                {
                    itemNum--;
                    continue;
                }
                thisPoint.X = (int)(((float)i) / (float)valuesNum * (endX - startX) + startX);
                thisPoint.Y = (int)(endY - (GetValue(itemNum) - minVal) / (maxVal - minVal) * (endY - startY));
                if (thisPoint.Y < (int)startY) thisPoint.Y = (int)startY;
                if (thisPoint.Y > (int)endY) thisPoint.Y = (int)endY;
                points.Add(thisPoint);
                itemNum--;
            }
            if (points.Count == 0) return;
            Point[] arrPoints = new Point[points.Count];
            points.CopyTo(arrPoints);
            Pen colorPen = new Pen(color);
            graph.DrawCurve(colorPen, arrPoints);
            colorPen.Dispose();
        }

        public override void SetAfterNew(int length, IValues values)
        {
            SmaLength = length;
            ComputeFor(values);
        }        
    }
}
