using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

namespace AutoBroker
{
    public class StockInfo
    {
        
        public string               Name;
        public string               RealName;
        public string               FileName;

        private List<StockValue>    fixing = null;        
        private StockStats          Statistics = null;

        public List<StockValue> Fixing
        {
            get
            {
                if (fixing == null) FillFixingData();
                return fixing;
            }
        }

        public StockStats GetStats()
        {
            if (Statistics == null)
            {
                Statistics = new StockStats(Fixing);
            }
            return Statistics;
        }

        public string GetFullName()
        {
            return RealName + " (" + Name + ")";
        }        

        public StockInfo(string filename)
        {
            FileName = filename;
            if (!File.Exists(FileName))
            {
                throw new ApplicationException("No file with name " + filename);
            }
            RealName = filename.Substring(FileName.LastIndexOfAny("/\\".ToCharArray())+1);
            RealName = RealName.Substring(0, RealName.IndexOf('.'));
            StreamReader reader = File.OpenText(FileName);
            string headerLine = (reader.ReadLine());
            headerLine = (reader.ReadLine());
            Name = headerLine.Split(";".ToCharArray())[0];
            reader.Close();
            fixing = null;
        }

        public void FillFixingData()
        {
            string[] lines = File.ReadAllLines(FileName);
            fixing = new List<StockValue>(lines.Length);
            for (int i = 1; i < lines.Length; i++)
            {
                fixing.Add(new StockValue(ref lines[i]));
            }
        }
    }
}
