using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace AutoBroker
{
    public partial class StockCartForm : Form
    {
        public TransactionHistory History = new TransactionHistory();
        public string FileName = "";

        public StockCartForm()
        {
            InitializeComponent();
        }

        public void LoadFromFile(string path)
        {
            FileName = path;
            if (File.Exists(path))
            {
                History.LoadFromFile(path);
            }
            UpdateCharts();
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            if (FileName != "")
            {
                History.SaveToFile(FileName);
            }
            Close();
        }

        private void buttonAddTransaction_Click(object sender, EventArgs e)
        {
            AddTransaction newTrans = new AddTransaction();
            newTrans.SetStocks(MainForm.GetInstance().LoadedStocks);
            if (newTrans.ShowDialog() == DialogResult.OK)
            {
                Transaction newTransaction = newTrans.GetResultTransaction();
                if (newTransaction!=null)
                {
                    History.AddTransaction(newTransaction);
                }
            }
            UpdateCharts();
        }

        private void UpdateCharts()
        {
            listViewHistory.Items.Clear();
            decimal saldo = 0.0m;
            decimal changes = 0.0m;
            foreach (Transaction trans in History.Transactions)
            {
                ListViewItem item = new ListViewItem();
                item.Text = trans.Time.ToShortDateString();
                item.SubItems.Add(new ListViewItem.ListViewSubItem(item, trans.StockName));
                item.SubItems.Add(new ListViewItem.ListViewSubItem(item, trans.SingleStockValue.ToFloat().ToString("F2")));
                item.SubItems.Add(new ListViewItem.ListViewSubItem(item, trans.StockQuantityDelta.ToString()));
                item.SubItems.Add(new ListViewItem.ListViewSubItem(item, trans.Provision.ToFloat().ToString("F2")));
                float gain = (-trans.StockQuantityDelta * trans.SingleStockValue.ToFloat()) - trans.Provision.ToFloat();
                changes += (decimal)Math.Abs(gain);
                saldo += (decimal)gain;
                item.SubItems.Add(new ListViewItem.ListViewSubItem(item, gain.ToString("F2")));
                listViewHistory.Items.Add(item);
            }
            listViewOwnedStocks.Items.Clear();
            labelHistoryOverall.Text = saldo.ToString("F2");
            labelChanges.Text = changes.ToString("F2");

            decimal worth = 0.0m;
            
            List<StockQuantity> stockList = History.GetCurrentlyOwnedStocks();
            foreach (StockQuantity stockQuan in stockList)
            {
                ListViewItem item = new ListViewItem(stockQuan.StockName);
                item.SubItems.Add(new ListViewItem.ListViewSubItem(item, stockQuan.Quantity.ToString()));
                StockInfo info = MainForm.GetInstance().GetStockInfoForName(stockQuan.StockName);
                if (info != null)
                {
                    float price = info.Fixing[info.Fixing.Count - 1].CloseValue.ToFloat();
                    price *= stockQuan.Quantity;
                    worth += (decimal)price;
                    item.SubItems.Add(new ListViewItem.ListViewSubItem(item, price.ToString("F2")));
                }
                listViewOwnedStocks.Items.Add(item);
            }
            labelWorth.Text = worth.ToString("F2");
            decimal worthWithProv = worth - (decimal)Transaction.ComputeNormalProvision((float)worth);
            if (worthWithProv < 0.0m) worthWithProv = 0.0m;
            labelWorthWithProv.Text = worthWithProv.ToString("F2");
            decimal worthProfit = (worthWithProv + saldo);
            labelWorthProfit.Text = worthProfit.ToString("F2");
            labelWorthProfitTaxed.Text = (worthProfit - (decimal)Transaction.ComputeTax((float)worthProfit)).ToString("F2");

            decimal transProfit = (decimal)History.GetTransactionProfit();
            labelTransProfit.Text = transProfit.ToString("F2");
            labelTransProfitTaxed.Text = (transProfit - (decimal)Transaction.ComputeTax((float)transProfit)).ToString("F2");


            
        }

        private void buttonRemoveTransaction_Click(object sender, EventArgs e)
        {
            if (listViewHistory.SelectedIndices.Count > 0)
            {
                History.Transactions.RemoveAt(listViewHistory.SelectedIndices[0]);
            }
            UpdateCharts();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

    }    
}