using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace AutoBroker
{
    public class TransactionHistory
    {
        public List<Transaction> Transactions = new List<Transaction>();

        public List<StockQuantity> GetCurrentlyOwnedStocks()
        {
            List<StockQuantity> ownedStocks = new List<StockQuantity>();
            Dictionary<string, int> ownedQuantities = new Dictionary<string, int>();

            foreach (Transaction trans in Transactions)
            {
                if (ownedQuantities.ContainsKey(trans.StockName))
                {
                    ownedQuantities[trans.StockName] =
                        ownedQuantities[trans.StockName] + trans.StockQuantityDelta;
                }
                else
                {
                    ownedQuantities.Add(trans.StockName, trans.StockQuantityDelta);
                }
            }
            foreach (KeyValuePair<string, int> stock in ownedQuantities)
            {
                if (stock.Value != 0)
                {
                    ownedStocks.Add(new StockQuantity(stock.Key, stock.Value));
                }
            }
            return ownedStocks;
        }

        public List<int> GetBuyIndicesForName(string name)
        {
            List<int> indices = new List<int>();
            for (int i = 0; i < Transactions.Count; i++)
            {                
                Transaction trans = Transactions[i];
                if ((name == trans.StockName) && (trans.StockQuantityDelta > 0))
                {
                    indices.Add(i);
                }
            }
            return (indices);
        }

        public float GetTransactionProfit()
        {
            decimal cashChange = 0.0m;
            Dictionary<string, int> ownedQuantities = new Dictionary<string, int>();
            int[] storedStocks = new int[Transactions.Count];
            for (int i=0; i < Transactions.Count; i++)
            {
                Transaction trans = Transactions[i];                
                if (trans.StockQuantityDelta > 0)
                {
                    storedStocks[i] = trans.StockQuantityDelta;
                }
            }
            for (int i = 0; i < Transactions.Count; i++)
            {
                Transaction trans = Transactions[i];                
                if (trans.StockQuantityDelta < 0)
                {
                    int toSell = - trans.StockQuantityDelta;
                    cashChange += (decimal)(((float)toSell) * trans.SingleStockValue.ToFloat() - trans.Provision.ToFloat());
                    List<int> buyIndices = GetBuyIndicesForName(trans.StockName);
                    foreach (int buyIndex in buyIndices)
                    {
                        Transaction buyTrans = Transactions[buyIndex];
                        int takeFromThis = toSell;
                        if (takeFromThis > storedStocks[buyIndex])
                        {
                            takeFromThis = storedStocks[buyIndex];
                        }
                        if (takeFromThis == 0) continue;                        
                        cashChange -= (decimal)(((float)takeFromThis) * buyTrans.SingleStockValue.ToFloat() + 
                            (((float)(takeFromThis))/(float) buyTrans.StockQuantityDelta) * buyTrans.Provision.ToFloat());
                        storedStocks[buyIndex] -= takeFromThis;
                        toSell -= takeFromThis;
                        if (toSell == 0) break;
                    }
                }
            }
            return (float) cashChange;
        }

        public void SaveToFile(string path)
        {
            if (File.Exists(path))
            {
                string backupPath = path.Substring(0, path.LastIndexOf('.')) + DateTime.Now.ToShortDateString() + ".bck";
                if (!File.Exists(backupPath))
                {
                    File.Move(path, backupPath);                    
                }                
            }
            StreamWriter writer = File.CreateText(path);
            if (writer == null) return;
            foreach (Transaction trans in Transactions)
            {
                trans.WriteToStream(writer);
            }
            writer.Close();
        }

        public void LoadFromFile(string path)
        {
            StreamReader reader = File.OpenText(path);
            if (reader == null) return;
            Transactions = new List<Transaction>();
            while (reader.Peek() > 0)
            {
                Transactions.Add(Transaction.FromStream(reader));
            }
            Transactions.Sort(new TransactionDateComparer());
            reader.Close();
        }

        public void AddTransaction(Transaction trans)
        {
            Transactions.Add(trans);
            Transactions.Sort(new TransactionDateComparer());
        }

    }
    public class TransactionDateComparer : IComparer<Transaction>
    {
        #region IComparer<Transaction> Members

        public int Compare(Transaction x, Transaction y)
        {
            return x.Time.CompareTo(y.Time);
        }

        #endregion
    }
}