using System;
using System.Collections.Generic;
using System.Data;
using System.Text;

namespace AutoBroker
{
    public class StockChartValues : IChartValues
    {
        private StockInfo Stock = null;
        private float minCloseValue;
        private float maxCloseValue;

        public StockChartValues(StockInfo stock)
        {
            Stock = stock;
            if (Stock != null)
            {
                minCloseValue = float.MaxValue;
                maxCloseValue = float.MinValue;
                foreach (StockValue curStock in Stock.Fixing)
                {
                    if (minCloseValue > curStock.CloseValue.ToFloat()) minCloseValue = curStock.CloseValue.ToFloat();
                    if (maxCloseValue < curStock.CloseValue.ToFloat()) maxCloseValue = curStock.CloseValue.ToFloat();
                }
            }
        }

        public int GetNumValues()
        {
            if (Stock == null) return 0;
            return Stock.Fixing.Count;
        }

        public float GetValue(int index)
        {
            if (Stock == null) return 0.0f;
            return Stock.Fixing[index].CloseValue.ToFloat();
        }

        public int GetVolume(int index)
        {
            if (Stock == null) return 0;
            return Stock.Fixing[index].Volume;
        }

        public int GetDecIntValue(int index)
        {
            if (Stock == null) return 0;
            return Stock.Fixing[index].CloseValue.ToDecInt();
        }

        public DateTime GetDate(int index)
        {
            if (Stock == null) return new DateTime();
            return Stock.Fixing[index].Date;
        }

        public void GetMinMaxValue(out float min, out float max)
        {
            min = minCloseValue;
            max = maxCloseValue;
        }

        public void GetMinMaxValueForLast(int lastCount, out float min, out float max)
        {
            min = new float();
            max = new float();
            if (Stock != null)
            {
                min = float.MaxValue;
                max = float.MinValue;
                for (int i = Stock.Fixing.Count - 1; (i >= 0) && (lastCount > 0); i--, lastCount--)
                {
                    StockValue curStock = Stock.Fixing[i];
                    if (min > curStock.CloseValue.ToFloat()) min = curStock.CloseValue.ToFloat();
                    if (max < curStock.CloseValue.ToFloat()) max = curStock.CloseValue.ToFloat();
                }
            }
        }

        public void GetMinMaxVolumeForLast(int lastCount, out int min, out int max)
        {
            min = new int();
            max = new int();
            if (Stock != null)
            {
                min = int.MaxValue;
                max = int.MinValue;
                for (int i = Stock.Fixing.Count - 1; (i >= 0) && (lastCount > 0); i--, lastCount--)
                {
                    StockValue curStock = Stock.Fixing[i];
                    if (min > curStock.Volume) min = curStock.Volume;
                    if (max < curStock.Volume) max = curStock.Volume;
                }
            }
        }

        public IValues Clone()
        {
            StockChartValues newVals = new StockChartValues(Stock);
            return newVals;
        }
    }
}
